--[[ simpleMinimap ]]--
local version = "10"
local smmSkins = {
  { square = false,
    texture = "Interface\\Minimap\\UI-Minimap-Border",
    mask = "textures\\MinimapMask",
  },
  { square = true,
    texture = "Interface\\AddOns\\simpleMinimap\\skins\\SquareMinimapBorder",
    mask = "Interface\\AddOns\\simpleMinimap\\skins\\SquareMinimapMask",
  },
}


--[[ initialize ]]--
--
-- debug / info messages
function smm_debug(text)
  if(ChatFrame2) then ChatFrame2:AddMessage(text,0,0.8,1.0) end
end
--
-- default settings
local function smm_setDefaults()
  smmConf = {
    version = version,
    cx = false,
    cy = false,
    alpha = 1,
    scale = 1,
    lock = false,
    skin = 1,
    showPings = true,
    buttons = {
      bgs = MiniMapBattlefieldFrame,
      meet = MiniMapMeetingStoneFrame,
      mail = MiniMapMailFrame,
      time = GameTimeFrame,
      track = MiniMapTrackingFrame,
      zoomin = MinimapZoomIn,
      zoomout = MinimapZoomOut,
    },
    pos = {
      bgs = 302,
      meet = 189,
      mail = 169,
      time = 137,
      track = 38,
      zoomin = 209,
      zoomout = 235,
    },
    hide = {
      location = false,
      time = false,
      zoom = false,
    },
  }
end
--
-- init & feedback for the XML load
function smm_onLoad()
  SLASH_SIMPMM1 = "/smm"
  SlashCmdList["SIMPMM"] = smm_slash
  this:RegisterEvent("VARIABLES_LOADED")
  this:RegisterEvent("PLAYER_ENTERING_WORLD")
  this:RegisterEvent("MINIMAP_PING")
  smm_debug(":: simpleMinimap version "..version.." loaded ::")
end
--
-- on event
function smm_onEvent()
  if(event=="MINIMAP_PING") then
    smm_minimapPing(arg1)
  elseif(event=="VARIABLES_LOADED") then
    if(not smmConf or smmConf.version < version) then
      smm_debug("smm :: configuration not present or outdated -- resetting")
      smm_setDefaults()
    end
    if(not CleanMinimap) then CleanMinimap = true end
  elseif(event=="PLAYER_ENTERING_WORLD") then
    MinimapCluster:SetMovable(true)
    Minimap:RegisterForDrag("LeftButton")
    Minimap:SetScript("OnDragStart",function() smm_minimapDrag(true) end)
    Minimap:SetScript("OnDragStop",function() smm_minimapDrag(false) end)
    MinimapZoneTextButton:RegisterForDrag("LeftButton")
    MinimapZoneTextButton:SetScript("OnDragStart",function() smm_minimapDrag(true) end)
    MinimapZoneTextButton:SetScript("OnDragStop",function() smm_minimapDrag(false) end)
    for _,f in pairs(smmConf.buttons) do
      f:SetMovable(true)
      f:RegisterForDrag("LeftButton")
      f:SetScript("OnDragStart",function() smm_buttonDrag(true) end)
      f:SetScript("OnDragStop",function() smm_buttonDrag(false) end)
    end
    smm_update()
  end
end


--[[ frame events ]]--
--
-- minimap zoom
function smm_minimapZoom(x)
  if(IsShiftKeyDown()) then
    local z=MinimapCluster:GetAlpha()
    if(x > 0) then
      if(z < 1) then smmConf.alpha = z + 0.05 else return end
    else
      if(z > 0) then smmConf.alpha = z - 0.05 else return end
    end
    MinimapCluster:SetAlpha(smmConf.alpha)
  else
    local z=Minimap:GetZoom()
    if(x > 0) then
      if(z < Minimap:GetZoomLevels()) then z = z + 1 end
    else
     if(z > 0) then z = z - 1 end
    end
    Minimap:SetZoom(z)
  end
end
--
-- minimap ping
function smm_minimapPing(x)
  MiniMapPing:SetScript("OnEnter",function() GameTooltip:SetOwner(this,"ANCHOR_TOPLEFT") GameTooltip:SetText("ping by |cFFFFFFCC"..UnitName(x)) GameTooltip:Show() end)
  MiniMapPing:SetScript("OnLeave",function() GameTooltip:Hide() end)
end
--
-- minimap mover
function smm_minimapDrag(x)
  if(x) then
    if(not smmConf.lock) then
      MinimapCluster.moving = true
      MinimapCluster:StartMoving()
     end
  elseif(MinimapCluster.moving) then
    MinimapCluster.moving = false
    MinimapCluster:StopMovingOrSizing()
    smmConf.cx,smmConf.cy = MinimapCluster:GetCenter()
  end
end
--
-- button mover
function smm_buttonDrag(x)
  local function getPos()
    local cx,cy = GetCursorPosition(UIParent)
    local mx,my = Minimap:GetLeft(),Minimap:GetBottom()
    local z = Minimap:GetEffectiveScale()
    return(math.deg(math.atan2(cy/z-my-70,mx-cx/z+70)))
  end
  if(x) then
    if(not smmConf.lock) then
      this.moving = true
      this:SetScript("OnUpdate",function()
        local x,y,z = 0,0,getPos()
        this:ClearAllPoints()
        if(smmSkins[smmConf.skin]["square"]) then
          x = math.max(-82,math.min(110*cos(z),84))
          y = math.max(-84,math.min(110*sin(z),82))
        else
          x = 81*cos(z)
          y = 81*sin(z)
        end
        this:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-x,y-52)
      end)
      this:StartMoving()
    end
  elseif(this.moving) then
    this.moving = false
    this:StopMovingOrSizing()
    this:SetScript("OnUpdate",nil)
    for n,f in pairs(smmConf.buttons) do
      if(f==this) then smmConf.pos[n] = getPos() end
    end
  end
end
--
-- minimap update
function smm_update()
  MinimapCluster:ClearAllPoints()
  if(smmConf.cx and smmConf.cy) then
    MinimapCluster:SetPoint("CENTER",UIParent,"BOTTOMLEFT",smmConf.cx,smmConf.cy)
  else
    MinimapCluster:SetPoint("TOPRIGHT",UIParent,"TOPRIGHT")
  end
  MinimapBorder:SetTexture(smmSkins[smmConf.skin]["texture"])
  Minimap:SetMaskTexture(smmSkins[smmConf.skin]["mask"])
  for n,f in pairs(smmConf.buttons) do
    local x,y = 0,0
    if(smmSkins[smmConf.skin]["square"]) then
      x = math.max(-82,math.min(110*cos(smmConf.pos[n]),84))
      y = math.max(-84,math.min(110*sin(smmConf.pos[n]),82))
    else
      x = 81*cos(smmConf.pos[n])
      y = 81*sin(smmConf.pos[n])
    end
    f:ClearAllPoints()
    f:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-x,y-52)
  end
  MinimapCluster:SetScale(smmConf.scale)
  if(smmConf.hide.location) then
    MinimapZoneTextButton:Disable()
    MinimapToggleButton:Disable()
    MinimapZoneTextButton:Hide()
    MinimapToggleButton:Hide()
    MinimapBorderTop:Hide()
  else
    MinimapZoneTextButton:Show()
    MinimapToggleButton:Show()
    MinimapBorderTop:Show()
    MinimapZoneTextButton:Enable()
    MinimapToggleButton:Enable()
  end
  if(smmConf.hide.time) then
    GameTimeFrame:Hide()
  else
    GameTimeFrame:Show()
  end
  if(smmConf.hide.zoom) then
    MinimapZoomIn:Disable()
    MinimapZoomOut:Disable()
    MinimapZoomIn:Hide()
    MinimapZoomOut:Hide()
  else
    MinimapZoomIn:Show()
    MinimapZoomOut:Show()
    MinimapZoomIn:Enable()
    MinimapZoomOut:Enable()
  end
  MinimapCluster:SetAlpha(smmConf.alpha)
  if(smmConf.showPings) then
    MiniMapPing:EnableMouse(true) else MiniMapPing:EnableMouse(false)
  end
end


--[[ slash command handler ]]--
--
function smm_slash(x)
  local _,_,y,z = string.find(x,"(%w+)%s*(.*)")
  if(y=="hide" and z) then
    if(smmConf.hide[z]==false) then
      smm_debug("smm :: hiding "..z)
      smmConf.hide[z]=true
    end
  elseif(y=="show" and z) then
    if(smmConf.hide[z]) then
      smm_debug("smm :: showing "..z)
      smmConf.hide[z]=false
    end
  elseif(y=="alpha" and z) then
    z=tonumber(z)
    if(z) then
      if (z<0) then z=0 elseif(z>1) then z=1 end
      smm_debug("smm :: alpha "..z)
      smmConf.alpha = z
    end
  elseif(y=="scale" and z) then
    z=tonumber(z)
    if(z) then
      if(z>0) then
        smm_debug("smm :: scale "..z)
        if(smmConf.cx and smmConf.cy) then
          smmConf.cx = (smmConf.scale / z) * smmConf.cx
          smmConf.cy = (smmConf.scale / z) * smmConf.cy
        end
        smmConf.scale = z
      end
    end
  elseif(y=="lock") then
    if(smmConf.lock) then
      smm_debug("smm :: unlocking minimap")
      smmConf.lock = false
    else
      smm_debug("smm :: locking minimap")
      smmConf.lock = true
    end
    return
  elseif(y=="read") then
    local cx,cy = MinimapCluster:GetCenter()
    smm_debug("cluster center offset :: "..cx..","..cy)
    smm_debug("minimap alpha :: "..Minimap:GetAlpha())
    smm_debug("minimap scale :: "..Minimap:GetScale())
    smm_debug("cluster alpha :: "..MinimapCluster:GetAlpha())
    smm_debug("cluster scale :: "..MinimapCluster:GetScale())
    for n,f in pairs(smmConf.buttons) do
      local cx,cy = f:GetCenter(UIParent)
      local mx,my = Minimap:GetLeft(),Minimap:GetBottom()
      local z = Minimap:GetEffectiveScale()
      smm_debug(n.." :: "..(math.deg(math.atan2(cy/z-my-70,mx-cx/z+70))))
    end
    return
  elseif(y=="skin") then
    smm_debug("smm :: changing the minimap skin")
    smmConf.skin = smmConf.skin + 1
    if(not smmSkins[smmConf.skin]) then smmConf.skin = 1 end
  elseif(y=="ping") then
    if(smmConf.showPings) then
      smm_debug("smm :: turning ping tooltips off")
      smmConf.showPings = false
    else
      smm_debug("smm :: turning ping tooltips on")
      smmConf.showPings = true
    end
  elseif(y=="reset") then
    smm_debug("smm :: resetting minimap")
    smm_setDefaults()
  else
    for _,t in pairs(sm_HELP) do
      DEFAULT_CHAT_FRAME:AddMessage(t,0.9,0.8,0)
    end
    return
  end
  smm_update()
end

